/*
* Base64 Encoder/Decoder
* (C) 1999-2010 Jack Lloyd
*
* Botan is released under the Simplified BSD License (see license.txt)
*/

#ifndef BOTAN_BASE64_FILTER_H_
#define BOTAN_BASE64_FILTER_H_

#include <botan/filters.h>
BOTAN_DEPRECATED_HEADER(b64_filt.h)

#endif
