/*
* ASN.1 OID
* (C) 1999-2007,2019 Jack Lloyd
*
* Botan is released under the Simplified BSD License (see license.txt)
*/

#ifndef BOTAN_ASN1_OID_H_
#define BOTAN_ASN1_OID_H_

#include <botan/asn1_obj.h>
BOTAN_DEPRECATED_HEADER(asn1_oid.h)

#endif
