/*
* Symmetric Key Length Specification
* (C) 2010 Jack Lloyd
*
* Botan is released under the Simplified BSD License (see license.txt)
*/

#ifndef BOTAN_KEY_LEN_SPECIFICATION_H_
#define BOTAN_KEY_LEN_SPECIFICATION_H_

#include <botan/sym_algo.h>
BOTAN_DEPRECATED_HEADER(key_spec.h)

#endif
