/*
* Filter interface for ciphers
* (C) 2013,2016 Jack Lloyd
*
* Botan is released under the Simplified BSD License (see license.txt)
*/

#ifndef BOTAN_CIPHER_FILTER_H_
#define BOTAN_CIPHER_FILTER_H_

#include <botan/filters.h>
BOTAN_DEPRECATED_HEADER(cipher_filter.h)

#endif
