/*
 * XMSS WOTS Public Key
 * (C) 2016,2017,2018 Matthias Gierlings
 *
 * Botan is released under the Simplified BSD License (see license.txt)
 **/

#ifndef BOTAN_XMSS_WOTS_PUBLICKEY_H_
#define BOTAN_XMSS_WOTS_PUBLICKEY_H_

#include <botan/xmss.h>
BOTAN_DEPRECATED_HEADER(xmss_wots_publickey.h)

#endif
