/*
* HOTP
* (C) 2017 Jack Lloyd
*
* Botan is released under the Simplified BSD License (see license.txt)
*/

#ifndef BOTAN_HOTP_H_
#define BOTAN_HOTP_H_

#include <botan/otp.h>
BOTAN_DEPRECATED_HEADER(hotp.h)

#endif
