/*
 * XMSS WOTS Parameters
 * (C) 2016,2018 Matthias Gierlings
 *
 * Botan is released under the Simplified BSD License (see license.txt)
 **/

#ifndef BOTAN_XMSS_WOTS_PARAMETERS_H_
#define BOTAN_XMSS_WOTS_PARAMETERS_H_

#include <botan/xmss_wots.h>
BOTAN_DEPRECATED_HEADER(xmss_wots_parameters.h)

#endif
