/*
* Basic Filters
* (C) 1999-2007 Jack Lloyd
* (C) 2013 Joel Low
*
* Botan is released under the Simplified BSD License (see license.txt)
*/

#ifndef BOTAN_BASEFILT_H_
#define BOTAN_BASEFILT_H_

// This header is deprecated and will be removed in a future major release

#include <botan/filters.h>

BOTAN_DEPRECATED_HEADER(basefilt.h)

#endif
