/*
* SM2 Encryption
* (C) 2017 Ribose Inc
*
* Botan is released under the Simplified BSD License (see license.txt)
*/

#ifndef BOTAN_SM2_ENC_KEY_H_
#define BOTAN_SM2_ENC_KEY_H_

#include <botan/sm2.h>

BOTAN_DEPRECATED_HEADER(sm2_enc.h)

#endif
