/*
* PKCS#11 Slot
* (C) 2016 Daniel Neus
* (C) 2016 Philipp Weber
*
* Botan is released under the Simplified BSD License (see license.txt)
*/

#ifndef BOTAN_P11_SLOT_H_
#define BOTAN_P11_SLOT_H_

#include <botan/p11_types.h>
BOTAN_DEPRECATED_HEADER(p11_slot.h)

#endif
