/*
* PKCS#11 Module
* (C) 2016 Daniel Neus, Sirrix AG
* (C) 2016 Philipp Weber, Sirrix AG
*
* Botan is released under the Simplified BSD License (see license.txt)
*/

#ifndef BOTAN_P11_MODULE_H_
#define BOTAN_P11_MODULE_H_

#include <botan/p11_types.h>
BOTAN_DEPRECATED_HEADER(p11_module.h)

#endif
