/*
* Botan is released under the Simplified BSD License (see license.txt)
*/

#ifndef BOTAN_X509_ALT_NAME_H_
#define BOTAN_X509_ALT_NAME_H_

#include <botan/pkix_types.h>
BOTAN_DEPRECATED_HEADER(asn1_alt_name.h)

#endif
