/*
* (C) 2017 Jack Lloyd
*
* Botan is released under the Simplified BSD License (see license.txt)
*/

#ifndef BOTAN_TOTP_H_
#define BOTAN_TOTP_H_

#include <botan/otp.h>
BOTAN_DEPRECATED_HEADER(totp.h)

#endif
