/*
* Keyed_Filter
* (C) 1999-2007 Jack Lloyd
*
* Botan is released under the Simplified BSD License (see license.txt)
*/

#ifndef BOTAN_KEYED_FILTER_H_
#define BOTAN_KEYED_FILTER_H_

#include <botan/filter.h>
BOTAN_DEPRECATED_HEADER(key_filt.h)

#endif
